//
// Copyright (c) 1994-1996, David McFadzean
//
// You may distribute under the terms of the Artistic License, 
// as specified in the README file.
//
// $Header: E:/simbiosys/RCS/CELWORLD.CPP 1.1 1996/06/26 10:54:56 dbm Exp $

#include "celworld.h"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// bioCellWorld
//
                
bioCellWorld::bioCellWorld (void)
{
	_cells = 0;
	_size = 0;
		
	_winWidth = 0;
	_winHeight = 0;
	
	_width = 0;
	_side = 0;
}

bioCellWorld::~bioCellWorld (void)
{
	for (int i = 0; i < _size; i++)
	{
		for (int j = 0; j < _size; j++)
		{
			delete _cells[i][j];
		}

		delete [_size] _cells[i];
	}

	delete [_size] _cells;
}

void
bioCellWorld::SetSize (int sz)
{                            
	_size = sz;
	
	_cells = new bioThing**[_size];
	
	ASSERT (_cells);
	
	for (int i = 0; i < _size; i++)
	{
		_cells[i] = new bioThing*[_size]; 
		
		ASSERT (_cells[i]);
		
		for (int j = 0; j < _size; j++)
		{
			_cells[i][j] = 0;
		}
	}
}
 
void
bioCellWorld::AddThing (bioThing* thing, double x, double y)
{
	int				i, j;

	bioWorld::AddThing (thing, x, y);

	i = (int) thing->GetY();
	j = (int) thing->GetX();
	
	// Only one thing may exist in a cell
	ASSERT (_cells[i][j] == 0);
		
	_cells[i][j] = thing;
}

void
bioCellWorld::RemoveThing (bioThing* thing)
{
	int				x = (int) thing->GetX();
	int				y = (int) thing->GetY();
	
	ASSERT (_cells[y][x] == thing);
	
//	Trace ("removing %p (%d, %d) class = %d\n", _cells[y][x], y, x, _cells[y][x]->ClassID());
	_cells[y][x] = 0;
	
	bioWorld::RemoveThing (thing);
}

void
bioCellWorld::PlaceRandom (bioThing* thing)
{        
	int				x, y;
	
	ASSERT (_size > 0);
	
	do
	{
		x = rand()%_size;
		y = rand()%_size;
	} while (_cells[y][x]);
	
	AddThing (thing, x, y);
	TurnThing (thing, 90.0 * (rand()%4));
}
 
#if 0 // MFC only
void
bioCellWorld::ReadMap (char* filename)
{
    CStdioFile	file (filename, CFile::modeRead);

    int			sz;
    int			i, j=0;
    char		foo[16];
    
    file.ReadString (foo, sizeof(foo));
    sz = atoi (foo);

    if (_size != sz) 
    {
		SetSize (sz);
    }

    char*	buffer = new char[2*_size];
    
    ASSERT (buffer);

    for (i = 0; i < _size; i++)
    {          
    	file.ReadString (buffer, 2*_size);
		// Trace (buffer);
		for (j = 0; j < _size; j++) 
		{
			CreateThing (buffer[j], j, i);
		}                             
    }

    delete buffer;
}
#endif

class bioNothing : public bioThing
{
  public:
  	override int		ClassID (void) {return 0;}
};

bioObject*
bioCellWorld::WhatsAt (double x, double y, bioThing* thing)
{
	int					i, j;
	double				nx, ny;
	static bioNothing	nothing;
	
	XformPos (x, y, thing, nx, ny);
	
	i = (int) ny;
	j = (int) nx;
	
	if (i < 0)
	{
		i += _size;
	}
	else if (i >= _size)
	{
		i -= _size;
	}	
	
	if (j < 0)
	{
		j += _size;
	}
	else if (j >= _size)
	{
		j -= _size;
	}    
	
	if (_cells[i][j])
	{
		return (bioObject*) _cells[i][j];
	}
	else
	{
		_SetPosition (&nothing, j, i);
		return (bioObject*) &nothing;
	}
} 

void
bioCellWorld::Resize (int width, int height)
{
	_winWidth = width;
	_winHeight = height;
	
	_side = min (width, height);
	_width = _side/_size;
	_side = _width*_size; 
	
	_offX = (_winWidth - _side)/2;
	_offY = (_winHeight - _side)/2;
}

void
bioCellWorld::UserSelect (int xx, int yy)
{
	int				x = (xx - _offX) / _width;
	int				y = (yy - _offY) / _width;
	
	if (x >= 0 && x < _size)
	{
		if (y >= 0 && y < _size)
		{
			_cells[y][x]->Select();
		}
	}		               
}
 
void
bioCellWorld::UserEdit (int xx, int yy)
{
	int				x = (xx - _offX) / _width;
	int				y = (yy - _offY) / _width;
	
	if (x >= 0 && x < _size)
	{
		if (y >= 0 && y < _size)
		{
			_cells[y][x]->Edit();
		}
	}		               
}


