//
// Copyright (c) 1994-1996, David McFadzean
//
// You may distribute under the terms of the Artistic License, 
// as specified in the README file.
//
// $Header: E:/simbiosys/RCS/GTYPE.CPP 1.1 1996/06/26 10:54:58 dbm Exp $

#include <stdlib.h> // for rand()
#include "gtype.h"    
#include "random.h"

////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//	bioGType
//

////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//	bioHapGType
//   

bioHapGType::bioHapGType (void)
{
	_bits = 0;
	_bytes = 0;
	_dna = 0;
}

bioHapGType::bioHapGType (bioHapGType& parent)
{
	_bits = 0;
	_bytes = 0;
	_dna = 0;

	(*this) = parent;
}

bioHapGType::~bioHapGType (void)
{
	delete _dna;
}
 
bioHapGType&
bioHapGType::operator= (const bioHapGType& rhs)
{        
	if (this == &rhs)
	{
		return *this;
	}
	
	if (_bytes != rhs._bytes)
	{
		if (_dna != 0)
		{
			delete _dna;
		}               

		_bits = rhs._bits;		                        
		_bytes = rhs._bytes;		                        
		_dna = new char[_bytes];  
		ASSERT (_dna);
	}

	for (int i = 0; i < _bytes; i++)
	{
		_dna[i] = rhs._dna[i];
	}
	
	return *this;
}


void
bioHapGType::SetLength (int length)
{
	assert (length > 0);
	assert (_dna == 0);
	
	_bits = length;
	_bytes = (_bits-1)/8 + 1;
	_dna = new char[_bytes];
	ASSERT (_dna);
}

int
bioHapGType::GetLength (void)
{
	return _bits;
}

void
bioHapGType::Randomize (void)
{             
	assert (_dna != 0);

	for (int i = 0; i < _bytes; i++)
	{
		_dna[i] = rand();
	}
}

char
bioHapGType::GetByte (int index)
{             
	assert (index >= 0);
	assert (index < _bytes);
	
	return _dna[index];
}

int
bioHapGType::GetBit (int index)
{         
	assert (index >= 0);
	assert (index < _bits);
  	
	char		byte = _dna[index/8];
	int			bit = index%8;

	return ((byte & (1<<bit)) > 0);
}

void
bioHapGType::SetBit (int index, int on)
{
	assert (index >= 0);
	assert (index < _bits);
  		
	char			mask = (1<<(index%8));

	if (on)
	{
		_dna[index/8] |= mask;
	}
	else
	{
		_dna[index/8] &= ~mask;
	}
}

void
bioHapGType::ResetBit (int index)
{
	SetBit (index, 0);
}

void
bioHapGType::ToggleBit (int index)
{   
	assert (index >= 0);
	assert (index < _bits);
  	
	SetBit (index, !GetBit(index));
}

void
bioHapGType::Mutate (double mrate)
{
	unsigned int	next;      	
	double			denom = log(1.0 - mrate);

	next = (int) ceil (log(DRand())/denom);                     
    
	while (next < (unsigned int)_bits)
	{                                  
		ToggleBit(next); 
		next += (int) ceil (log(DRand())/denom);              
	}
}

bioGType*
bioHapGType::Crossover (bioGType* other)
{
	bioHapGType&	parent = *(bioHapGType*) other; 
	bioHapGType*	kid = new bioHapGType (parent);       
	
	ASSERT (kid);
	
	int				x1 = rand() % _bits;
	int				x2 = rand() % (_bits-1);
	int				i;

	if (x2 >= x1) 
	{
		x2++;
	}
	else
	{
		i = x1;
		x1 = x2;
		x2 = i;
	}
     
	for (i = x1; i < x2; i++)
	{
		kid->SetBit (i, GetBit(i));
	}
	
	return kid;
}
  

