//
// Copyright (c) 1994-1996, David McFadzean
//
// You may distribute under the terms of the Artistic License, 
// as specified in the README file.
//
// $Header: E:/simbiosys/RCS/SIMULAT.CPP 1.1 1996/06/26 10:55:02 dbm Exp $

#include "simulat.h"

////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//	bioSimulation
//
   
bioSimulation::bioSimulation (void)
{
	_world = 0;      
	_curAct = 0;
	_totalAct = 0;
	_curEnv = 0;
	_totalEnv = 0;
	_curGen = 0;
}
  
bioSimulation::~bioSimulation (void)
{ 
	delete _world;
	_populations.DeleteAll();
}
 
void
bioSimulation::SetWorld (bioWorld* world)
{
	assert (_world == 0);
	_world = world;
} 

bioWorld*
bioSimulation::GetWorld (void)
{
	return _world;
}

void
bioSimulation::AddPopulation (bioPopulation* pop)
{
	_populations.Append (pop);
}               

int
bioSimulation::GetData (int src, void* dest)
{
	switch (src)
	{      
		case 1:
			*(int*) dest = _curAct;
			break;
			
		case 2:
			*(int*) dest = _totalAct;
			break;
			
		case 3:
			*(int*) dest = _curEnv;
			break;
			
		case 4:
			*(int*) dest = _totalEnv;
			break;
			
		case 5:
			*(int*) dest = _curGen;
			break; 
			
		default:
			return 0;
	}
	
	return 1;
}

int
bioSimulation::NextStep (void)
{           
	int 				rc = 0;
	
	if (StartEnvCycle())
	{	
		if (StartBreedCycle())
		{
			DoBreedCycle(); 
	 		++rc;	
		}
		 
		DoEnvCycle();		
		++rc;
	}        
	 
	DoActionCycle();		
	++rc;
	
 	return rc;         
}

void
bioSimulation::DoBreedCycle (void)
{
	//Trace ("\n\n***** Breed Cycle\n");        
	bioPopulation*		pop;
	 		
	for (_populations.First(); pop = (bioPopulation*) _populations.Next(); )
	{
		ASSERT_VALID(pop);
	 			
		pop->Breed();
	}
	 		
	_curEnv = 0;
	++_curGen;
}

void
bioSimulation::DoEnvCycle (void)
{
	//Trace ("\n\n***** Environ Cycle %d\n", _curEnv);        
		                       
	_curAct = 0;
	++_curEnv;
	++_totalEnv;
}

void
bioSimulation::DoActionCycle (void)
{
	//Trace ("\n\n***** Action Cycle\n"); 
	       
	_world->NextStep();
	 	
	++_curAct;
	++_totalAct;
}

